@@ -14,6 +14,8 @@ import qualified Data.Text                  as T
 import qualified Data.Text.Encoding         as TE
 import qualified Data.Text.Lazy             as L
 import           Data.Time.Compat
+import           Data.Time.Calendar.Month.Compat (Month)
+import           Data.Time.Calendar.Quarter.Compat (Quarter, QuarterOfYear (..))
 import qualified Data.UUID.Types            as UUID
 import           Data.Version
 import           Data.Word
@@ -87,6 +89,10 @@ spec = do
     checkUrlPiece  (Proxy :: Proxy UTCTime)   "UTCTime"
     checkUrlPiece  (Proxy :: Proxy NominalDiffTime) "NominalDiffTime"
     checkUrlPiece  (Proxy :: Proxy DayOfWeek) "DayOfWeek"
+    checkUrlPiece  (Proxy :: Proxy Quarter)   "Quarter"
+    checkUrlPiece  (Proxy :: Proxy QuarterOfYear) "QuarterOfYear"
+    checkUrlPiece  (Proxy :: Proxy Month)     "Month"
+    checkUrlPiece  (Proxy :: Proxy DayOfWeek) "DayOfWeek"
     checkUrlPiece  (Proxy :: Proxy Version)   "Version"
     checkUrlPiece  (Proxy :: Proxy UUID.UUID) "UUID"
     checkUrlPiece' setCookieGen               "Cookie"
